# frozen_string_literal: true

# Return Hermes::Applicant attributes from parsed xml

module Hermes
  module BoClient
    module Mappings
      class ApplicantReverseMap
        include Hermes::BoClient::MappingHelper
        
        def initialize(xml, is_main_applicant, coapplicant_index = nil, is_colocataire=false)
          @xml                = xml
          @is_main_applicant  = is_main_applicant
          @coapplicant_index  = coapplicant_index
          @is_colocataire  = is_colocataire
        end

        def build_applicant
          demande_logement = @xml["interfaceNuu"]["demande"]["demandeLogement"] #["personnePhysique"]["demandeur"]["mel"]
          personnePhysique = demande_logement["personnePhysique"]
          situation = personnePhysique["situation"] || {}
      
          
          if @is_main_applicant
            demandeur = personnePhysique["demandeur"]
          else
            if @is_colocataire
              demandeur = (personnePhysique["listeColocataire"]["colocataire"].is_a? Array) ? personnePhysique["listeColocataire"]["colocataire"][@coapplicant_index]: personnePhysique["listeColocataire"]["colocataire"]
            else
              demandeur = (personnePhysique["listeCodemandeur"]["codemandeur"].is_a? Array) ? personnePhysique["listeCodemandeur"]["codemandeur"][@coapplicant_index] : personnePhysique["listeCodemandeur"]["codemandeur"]
            end
          end

          # On récupère les valeurs du sne et on vérifie si les années correspondent a N-1 et N-2 de l'année courante
          # On sauvegardera les valeurs de montantMoins1 et montantMoins2 qu'en cas de correspondance des années fiscales
         
          year_minus_one            = demandeur["revenuFiscal"]&.fetch("anneeMoins1", nil)
         
          valid_year_minus_one      = (DateTime.current.year - 1).to_s
          
          has_valid_year_minus_one  = year_minus_one&.to_s == valid_year_minus_one
          year_minus_two            = demandeur["revenuFiscal"]&.fetch("anneeMoins2", nil)
          valid_year_minus_two      = (DateTime.current.year - 2).to_s
          has_valid_year_minus_two  = year_minus_two&.to_s == valid_year_minus_two
          amount_year_minus_one     = has_valid_year_minus_one ? demandeur["revenuFiscal"]&.fetch("montantMoins1", nil) : nil
          amount_year_minus_two     = has_valid_year_minus_two ? demandeur["revenuFiscal"]&.fetch("montantMoins2", nil) : nil
          # Si nous somme en 2020 (N-1 = 2019 et N-2 = 2018) et que nous avons sur le sne (N-1 = 2018 et N-2 = 2017) alors on attribut le montant de N-1 du sne à N-2 en base
          unless has_valid_year_minus_two
            amount_year_minus_two = demandeur["revenuFiscal"]&.fetch("montantMoins1", nil) if year_minus_one&.to_s == valid_year_minus_two
          end

          attributes = {
            nationality: demandeur["nationalite"]&.fetch("code", nil),
            civility: demandeur["civilite"]&.fetch("code", nil),
            birth_date: Date.parse(demandeur["dateNaissance"]),
            homephone: demandeur["telDomicile"],
            workphone: demandeur["telTravail"],
            cellphone: demandeur["telPortable"],
            has_other_employers: demandeur["situationProfessionnelle"]&.fetch("multiEmployeur", nil),
            year_minus_one: valid_year_minus_one,
            amount_year_minus_one: amount_year_minus_one,
            year_minus_two: valid_year_minus_two,
            amount_year_minus_two: amount_year_minus_two,
            family_situation: demandeur["situationFamiliale"]&.fetch("code", nil),
            email_advances_assistant: demandeur['melAssistantDemarches'],
            #10095 ajout de nouveaux champs cerfa v4
            unemployment_period_date_end: demandeur["situationProfessionnelle"]&.fetch("dateFinPeriodeChomage", nil),
            unemployment_period: demandeur["situationProfessionnelle"]&.fetch("periodeChomage", nil),
            pro_or_home_tel_number: demandeur["telDomicilePro"],
            command_to_leave_date: situation&.fetch("compDateCommandement", nil),
            has_valid_year_minus_two: has_valid_year_minus_two
          }
          if @is_main_applicant
            # datas actual_housing_details : adresseLogement + logementActuel
            attributes[:actual_housing_details] = {}
            adresseLogement = personnePhysique["adresseLogement"] || {}
            attributes[:actual_housing_details]['foreign_address'] = adresseLogement["etranger"]
            attributes[:actual_housing_details]['appartment'] = adresseLogement["appartement"]
            attributes[:actual_housing_details]['floor'] = adresseLogement["etage"]
            attributes[:actual_housing_details]['stair'] = adresseLogement["escalier"]
            attributes[:actual_housing_details]['building'] = adresseLogement["batiment"]
            attributes[:actual_housing_details]['company_adress'] = adresseLogement["compAdresse"]
            attributes[:actual_housing_details]['known_place'] = adresseLogement["lieuDit"]
            attributes[:actual_housing_details]['number'] = adresseLogement["numero"]
            attributes[:actual_housing_details]['way'] = adresseLogement["voie"]
            attributes[:actual_housing_details]['postal_code'] = adresseLogement["codePostal"]
            attributes[:actual_housing_details]['host_name'] = adresseLogement["nomHebergeant"]
            attributes[:actual_housing_details]['foreign_postal_code'] = adresseLogement["codePostalEtranger"]
            attributes[:actual_housing_details]['foreign_district'] = adresseLogement["communeEtranger"]
            attributes[:actual_housing_details]['country'] = adresseLogement["pays"]
            attributes[:actual_housing_details]['address_insee_code'] = adresseLogement["commune"]&.fetch("code", nil)

            situation = personnePhysique["situation"] || {}
            attributes[:actual_housing_details]['monthly_rental_amount'] = situation["montantMensuelLoyer"].to_s
            attributes[:actual_housing_details]['APL_monthly_amount'] = situation["montantMensuelAPL"].to_s
            attributes[:actual_housing_details]['type'] = owned_housing_type_v4_map(situation["nombrePiecesActuel"]&.fetch("code", nil))
            attributes[:actual_housing_details]['number_of_resident'] = situation["nombreHabitant"]
            attributes[:actual_housing_details]['area'] = situation["surface"]
            attributes[:actual_housing_details]['housing_category_list'] = situation["categorieLogement"]&.fetch("code", nil)
            attributes[:actual_housing_details]['housing_surface'] = situation["surface"]
            attributes[:actual_housing_details]['is_house_owner'] = situation["proprietaire"]
            attributes[:actual_housing_details]['typeLogementActuel'] = owned_housing_type_v4_map(situation["nombrePiecesActuel"]&.fetch("code", nil))
            
            # datas owned_housing_details
            attributes[:owned_housing_details] = {}
            if situation['proprietaire'] && situation['proprietaire'].to_bool
              attributes[:owned_housing_details] = {}
              attributes[:owned_housing_details]["foreign_address"] = situation['etranger']
              attributes[:owned_housing_details]["postal_code"] = situation['codePostal']
              attributes[:owned_housing_details]['address_insee_code'] = situation["commune"]&.fetch("code", nil)
              attributes[:owned_housing_details]['foreign_postal_code'] = situation["codePostalEtranger"]
              attributes[:owned_housing_details]['foreign_district'] = situation["communeEtranger"]
              attributes[:owned_housing_details]['country'] = situation["pays"]
              attributes[:owned_housing_details]['type'] = owned_housing_type_v4_map(situation["nombrePiecesPossedees"]&.fetch("code", nil))
            
            end

            # datas postal_details
            attributes[:postal_details] = {}
            adressePostale = demandeur["adressePostale"] || {}
            attributes[:postal_details]['etranger'] = adressePostale["etranger"]
            attributes[:postal_details]['appartment_number'] = adressePostale["appartement"]
            attributes[:postal_details]['appartment_floor'] = adressePostale["etage"]
            attributes[:postal_details]['appartment_staircase'] = adressePostale["escalier"]
            attributes[:postal_details]['appartment_building'] = adressePostale["batiment"]
            attributes[:postal_details]['address_add_address'] = adressePostale["compAdresse"]
            attributes[:postal_details]['address_number'] = adressePostale["numero"]
            attributes[:postal_details]['address_street'] = adressePostale["voie"]
            attributes[:postal_details]['appartment_address_place'] = adressePostale["lieuDit"]
            attributes[:postal_details]['postal_code'] = adressePostale["codePostal"]
            attributes[:postal_details]['current_address_postal_code'] = adressePostale["codePostalEtranger"]
            attributes[:postal_details]['foreign_city'] = adressePostale["communeEtranger"]
            attributes[:postal_details]['foreign_country'] = adressePostale["pays"]
            attributes[:postal_details]['houser_name'] = adressePostale["nomHebergeant"]
            attributes[:postal_details]['address_insee_code'] = adressePostale["commune"]&.fetch("code", nil)
            
          else
            attributes[:temporary_id] = demandeur['identifiantTemporaire']
          end

          attributes[:ss_id] = demandeur["nir"]
          attributes[:first_name] = demandeur['prenoms']
          attributes[:last_name] = demandeur["nom"] || demandeur["nomPatronymique"]
          attributes[:patronymic_name] = demandeur["nomPatronymique"]
          attributes[:email] = demandeur["mel"]
          attributes[:sms_notification] = (demandeur["notifSms"] != "false") if demandeur["notifSms"]
          attributes[:event_notification] = (demandeur["notifEvt"] != "false") if demandeur["notifEvt"]
          attributes[:work_contract_type] = typeContratTravail_v4_map(
            demandeur["situationProfessionnelle"]&.fetch("typeContratTravail", nil)&.fetch("code", nil), 
            demandeur['situationProfessionnelle']&.fetch('cddInterim', nil),
            demandeur['situationProfessionnelle']&.fetch('siretEmployeur', nil)
          )

          attributes[:monthly_incomes] = {}
          demandeur['listeRessourceRecue'] = transform_in_array(demandeur['listeRessourceRecue'])
          demandeur['listeRessourceRecue']&.each do |ressource|
            if ressource
              detailRessource = transform_in_array(ressource["detailRessource"])
              detailRessource.each do |detail|
                attributes[:monthly_incomes][listeRessourceRecue_v4_map(detail["ressource"]&.fetch("code", nil).to_s)] = detail["montant"].to_i
              end
            end
          end

          attributes[:monthly_charges] = {}
          demandeur['listeRessourceVersee'] = transform_in_array(demandeur['listeRessourceVersee'])
          demandeur['listeRessourceVersee']&.each do |ressource|
            if ressource
              detailRessource = transform_in_array(ressource["detailRessource"])
              detailRessource.each do |detail|
                attributes[:monthly_charges][listeRessourceVersee_v4_map(detail["ressource"]&.fetch("code", nil).to_s)] = detail["montant"].to_i
              end
            end
          end

          attributes[:profession_details] = {}
          if demandeur['situationProfessionnelle']
            if demandeur['situationProfessionnelle']['etranger']
              if (demandeur['situationProfessionnelle']['etranger'] == "false")
                attributes[:profession_details]["foreign"] = false
              else
                attributes[:profession_details]["foreign"] = true
              end
            end
          
            attributes[:profession_details]["profession"] = demandeur['situationProfessionnelle']['profession']
            attributes[:workplace_postal_code] = demandeur['situationProfessionnelle']['codePostal']
            attributes[:foreign_workplace_postal_code] = demandeur['situationProfessionnelle']['codePostalEtranger']
            attributes[:foreign_workplace_locality] = demandeur['situationProfessionnelle']['communeEtranger']
            attributes[:foreign_workplace_country] = demandeur['situationProfessionnelle']['pays']
            attributes[:workplace_insee_code] = demandeur['situationProfessionnelle']['commune']&.fetch("code", nil).to_s
            attributes[:main_employer_name] = demandeur['situationProfessionnelle']['nomEmployeurPrincipal']
            attributes[:main_employer_siret] = demandeur['situationProfessionnelle']['siretEmployeur']
          end

          attributes
        end
        def listeRessourceRecue_v4_map(code)
          map = {
            "ASPA"=>"AMV",
            "AUTR"=>"AUT"
          }
          return map[code] || code
        end
        def listeRessourceVersee_v4_map(code)
          map = {
            "PAVS"=>"PAV"
          }
          return map[code] || code
        end
        def typeContratTravail_v4_map(code, cdd_interim, siret)
          map = {
            "SAL"=> cdd_interim == "true" ? "CDIP" : "CDI",
           "AGE"=>"AGP",
           "ETA"=> siret ? "APP" : "ETU",
           "AUTS"=>"AUT",
           "ASS"=> "AUT"
           
          }
          return map[code] || code
        end
        def owned_housing_type_v4_map(code)
          map = {
            "P1"=>"T1",
            "P2"=>"T2",
            "P3"=>"T3",
            "P4"=>"T4",
            "P5"=>"T5",
            "P6P"=>"T6P"
          }
          return map[code] || code
        end
      end
    end
  end
end
