def send_doc_to_sne(csv_output_path)
  require 'csv'
  output_data = []

  pieces = Hermes::RequiredDocument.where(sne_piece_id: nil,
                                          :sne_code.nin => ["", nil],
                                          :doc_id.nin => ["", nil],
                                          :sne_category.nin => ["", nil],
                                          :sne_piece_label.nin => ["", nil],
                                          :applicant_id.nin => ["", nil]
  )
  pieces.batch_size(5).each_with_index do |piece, i|
    begin
      if (piece.doc_id && piece.applicant)
        should_be_sended_to_sne = true
        parent_folder_ids = piece.folder_where_is_required_for_nud + piece.folder_where_is_required_for_consultation + piece.folder_where_is_required_for_application
        parent_folder_ids.flatten.uniq.compact.each do |folder_id|
          parent_folder = Hermes::HousingRequestFolder.find(folder_id)
          active_application = parent_folder&.housing_request&.active_application
          if active_application&.status == "TRANSMIS BAILLEUR"
            should_be_sended_to_sne = false
          end
        end
        
        if should_be_sended_to_sne
          piece.upload_file_to_sne
          piece.save
          output_data.push({
                             sended_piece_id: piece.id.to_s,
                             error: false
                           })
        
        end
      end
      
    rescue Exception => e
      output_data.push({
                         sended_piece_id: piece.id.to_s,
                         error: e.to_s
                       })
    end
  end
  return if output_data.length == 0
  
  CSV.open(csv_output_path, "wb") do |csv|
    csv << output_data.first.keys # adds the attributes name on the first line
    output_data.each do |hash|
      csv << hash.values
    end
  end
end

send_doc_to_sne("/home/webapp/output_send_doc_to_sne.csv")
